@@ -27,43 +27,31 @@ namespace GtkSharp.Generation {
 	using System.IO;
 	using System.Xml;
 
-	public class Method  {
+	public class Method : MethodBase  {
 		
-		private string libname;
-		private XmlElement elem;
-		private Parameters parms;
-		private Signature sig;
-		private ImportSignature isig;
-		private MethodBody body;
-		private ClassBase container_type;
+		private ReturnValue retval;
 
 		private bool initialized = false;
 		private string call;
-		private string rettype, m_ret, s_ret;
-		private string element_type = null;
-		private string name, cname, safety;
+		private string name;
 		private string protection = "public";
 		private bool is_get, is_set;
-		private bool needs_ref = false;
+		private bool deprecated = false;
 
-		public Method (string libname, XmlElement elem, ClassBase container_type) 
+		public Method (XmlElement elem, ClassBase container_type) : base (elem, container_type)
 		{
-			this.elem = elem;
-			if (elem["parameters"] != null) {
-				parms = new Parameters (elem["parameters"], container_type.NS);
-			}
-			this.container_type = container_type;
+			this.retval = new ReturnValue (elem["return-type"]);
+			if (!container_type.IsDeprecated && elem.HasAttribute ("deprecated"))
+				deprecated = elem.GetAttribute ("deprecated") == "1";
 			this.name = elem.GetAttribute("name");
 			if (name == "GetType")
 				name = "GetGType";
-			if (elem.HasAttribute ("library"))
-				this.libname = elem.GetAttribute ("library");
-			else
-				this.libname = libname;
-			
-			// caller does not own reference?
-			if (elem.HasAttribute ("needs_ref"))
-				this.needs_ref = (elem.GetAttribute ("needs_ref") == "1");
+		}
+
+		public bool IsDeprecated {
+			get {
+				return deprecated;
+			}
 		}
 
 		public bool IsGetter {
@@ -78,12 +66,6 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		private bool IsShared {
-			get {
-				return elem.HasAttribute("shared");
-			}
-		}
-
 		public string Name {
 			get {
 				return name;
@@ -104,13 +86,7 @@ namespace GtkSharp.Generation {
 
 		public string ReturnType {
 			get {
-				return s_ret;
-			}
-		}
-
-		public Signature Signature {
-			get {
-				return sig;
+				return retval.CSType;
 			}
 		}
 
@@ -133,67 +109,36 @@ namespace GtkSharp.Generation {
 			return (a.Signature.Types == b.Signature.Types);
 		}
 
+		public override int GetHashCode ()
+		{
+			return Name.GetHashCode () ^ (Signature == null ? 0 : Signature.Types.GetHashCode ());
+		}
+
 		private bool Initialize ()
 		{
 			if (initialized)
 				return true;
 
-			if (parms != null && !parms.Validate ()) {
-				Console.Write ("in method " + Name + " ");
-				return false;
-			}
-
-			XmlElement ret_elem = elem["return-type"];
-			if (ret_elem == null) {
-				Console.Write("Missing return type in method " + Name + " ");
-				Statistics.ThrottledCount++;
-				return false;
-			}
+			Parameters parms = Parameters;
+			is_get = ((((parms.IsAccessor && retval.CSType == "void") || (parms.Count == 0 && retval.CSType != "void")) || (parms.Count == 0 && retval.CSType != "void")) && Name.Length > 3 && (Name.StartsWith ("Get") || Name.StartsWith ("Is") || Name.StartsWith ("Has")));
+			is_set = ((parms.IsAccessor || (parms.Count == 1 && retval.CSType == "void")) && (Name.Length > 3 && Name.Substring(0, 3) == "Set"));
 			
-			SymbolTable table = SymbolTable.Table;
-
-			rettype = ret_elem.GetAttribute("type");
-			m_ret = table.GetMarshalReturnType(rettype);
-			s_ret = table.GetCSType(rettype);
-			cname = elem.GetAttribute("cname");
-			if (ret_elem.HasAttribute("element_type"))
-				element_type = ret_elem.GetAttribute("element_type");
-			
-			if (ret_elem.HasAttribute("array")) {
-					s_ret += "[]";
-					m_ret += "[]";
-				}
-
-			is_get = (((parms != null && ((parms.IsAccessor && s_ret == "void") || (parms.Count == 0 && s_ret != "void"))) || (parms == null && s_ret != "void")) && Name.Length > 3 && (Name.StartsWith ("Get") || Name.StartsWith ("Is") || Name.StartsWith ("Has")));
-			is_set = ((parms != null && (parms.IsAccessor || (parms.Count == 1 && s_ret == "void"))) && (Name.Length > 3 && Name.Substring(0, 3) == "Set"));
-			
-			if (parms != null)
-				parms.Static = IsShared;
-
-			sig = new Signature (parms);
-			isig = new ImportSignature (parms, container_type.NS);
-			body = new MethodBody (parms, container_type.NS);
-			call = "(" + (IsShared ? "" : container_type.CallByName () + (parms != null ? ", " : "")) + body.GetCallString (is_set) + ")";
-
-			if (body.ThrowsException && !(container_type is InterfaceGen))
-				safety = "unsafe ";
-			else
-				safety = "";
+			call = "(" + (IsStatic ? "" : container_type.CallByName () + (parms.Count > 0 ? ", " : "")) + Body.GetCallString (is_set) + ")";
 
 			initialized = true;
 			return true;
 		}
 		
-		public bool Validate ()
+		public override bool Validate ()
 		{
-			if (!Initialize ())
+			if (!Initialize () || !base.Validate ())
 				return false;
 
-			if (m_ret == "" || s_ret == "") {
-				Console.Write("rettype: " + rettype + " in method " + Name + " ");
-				Statistics.ThrottledCount++;
+			if (!retval.Validate ()) {
+				Console.Write(" in method " + Name + " ");
 				return false;
 			}
+
 			return true;
 		}
 		
@@ -205,31 +150,38 @@ namespace GtkSharp.Generation {
 			else
 				complement = 'G';
 			
-			return container_type.GetMethod (complement + elem.GetAttribute("name").Substring (1));
+			return container_type.GetMethod (complement + name.Substring (1));
 		}
 		
+		public string Declaration {
+			get {
+				return retval.CSType + " " + Name + " (" + (Signature != null ? Signature.ToString() : "") + ");";
+			}
+		}
+
 		private void GenerateDeclCommon (StreamWriter sw, ClassBase implementor)
 		{
-			if (elem.HasAttribute("shared"))
+			if (IsStatic)
 				sw.Write("static ");
-			sw.Write(safety);
+			sw.Write (Safety);
 			Method dup = null;
 			if (container_type != null)
 				dup = container_type.GetMethodRecursively (Name);
 			if (implementor != null)
 				dup = implementor.GetMethodRecursively (Name);
 
-			if (Name == "ToString" && parms == null)
+			if (Name == "ToString" && Parameters.Count == 0)
 				sw.Write("override ");
 			else if (Name == "GetGType" && container_type is ObjectGen)
 				sw.Write("new ");
-			else if (elem.HasAttribute("new_flag") || (dup != null && dup.Initialize () && ((dup.Signature != null && sig != null && dup.Signature.ToString() == sig.ToString()) || (dup.Signature == null && sig == null))))
+			else if (Modifiers == "new " || (dup != null && dup.Initialize () && ((dup.Signature != null && Signature != null && dup.Signature.ToString() == Signature.ToString()) || (dup.Signature == null && Signature == null))))
 				sw.Write("new ");
 
 			if (is_get || is_set) {
-				if (s_ret == "void")
-					s_ret = parms.AccessorReturnType;
-				sw.Write(s_ret);
+				if (retval.CSType == "void")
+					sw.Write (Parameters.AccessorReturnType);
+				else
+					sw.Write(retval.CSType);
 				sw.Write(" ");
 				if (Name.StartsWith ("Get") || Name.StartsWith ("Set"))
 					sw.Write (Name.Substring (3));
@@ -237,9 +189,9 @@ namespace GtkSharp.Generation {
 					sw.Write (Name);
 				sw.WriteLine(" { ");
 			} else if (IsAccessor) {
-				sw.Write (sig.AccessorType + " " + Name + "(" + sig.AsAccessor + ")");
+				sw.Write (Signature.AccessorType + " " + Name + "(" + Signature.AsAccessor + ")");
 			} else {
-				sw.Write(s_ret + " " + Name + "(" + (sig != null ? sig.ToString() : "") + ")");
+				sw.Write(retval.CSType + " " + Name + "(" + (Signature != null ? Signature.ToString() : "") + ")");
 			}
 		}
 
@@ -248,7 +200,7 @@ namespace GtkSharp.Generation {
 			if (!Initialize ()) 
 				return;
 
-			if (elem.HasAttribute("shared"))
+			if (IsStatic)
 				return;
 
 			if (is_get || is_set)
@@ -282,14 +234,14 @@ namespace GtkSharp.Generation {
 
 		public void GenerateImport (StreamWriter sw)
 		{
-			string import_sig = IsShared ? "" : container_type.MarshalType + " raw";
-			import_sig += !IsShared && parms != null ? ", " : "";
-			import_sig += isig.ToString();
-			sw.WriteLine("\t\t[DllImport(\"" + libname + "\")]");
-			if (m_ret.StartsWith ("[return:"))
-				sw.WriteLine("\t\t" + m_ret + " static extern " + safety + s_ret + " " + cname + "(" + import_sig + ");");
+			string import_sig = IsStatic ? "" : container_type.MarshalType + " raw";
+			import_sig += !IsStatic && Parameters.Count > 0 ? ", " : "";
+			import_sig += ImportSignature.ToString();
+			sw.WriteLine("\t\t[DllImport(\"" + LibraryName + "\")]");
+			if (retval.MarshalType.StartsWith ("[return:"))
+				sw.WriteLine("\t\t" + retval.MarshalType + " static extern " + Safety + retval.CSType + " " + CName + "(" + import_sig + ");");
 			else
-				sw.WriteLine("\t\tstatic extern " + safety + m_ret + " " + cname + "(" + import_sig + ");");
+				sw.WriteLine("\t\tstatic extern " + Safety + retval.MarshalType + " " + CName + "(" + import_sig + ");");
 			sw.WriteLine();
 		}
 
@@ -300,21 +252,21 @@ namespace GtkSharp.Generation {
 			if (!Initialize ()) 
 				return;
 
-			if (implementor != null && elem.HasAttribute("shared"))
+			if (implementor != null && IsStatic)
 				return;
 
 			/* we are generated by the get Method, if there is one */
 			if (is_set || is_get)
 			{
-				if (!elem.HasAttribute("new_flag") && container_type.GetPropertyRecursively (Name.Substring (3)) != null)
+				if (Modifiers != "new " && container_type.GetPropertyRecursively (Name.Substring (3)) != null)
 					return;
 				comp = GetComplement ();
-				if (comp != null && comp.Validate () && is_set && parms.AccessorReturnType == comp.s_ret)
+				if (comp != null && comp.Validate () && is_set && Parameters.AccessorReturnType == comp.ReturnType)
 					return;
-				if (comp != null && is_set && parms.AccessorReturnType != comp.s_ret)
+				if (comp != null && is_set && Parameters.AccessorReturnType != comp.ReturnType)
 				{
 					is_set = false;
-					call = "(Handle, " + body.GetCallString (false) + ")";
+					call = "(Handle, " + Body.GetCallString (false) + ")";
 					comp = null;
 				}
 				/* some setters take more than one arg */
@@ -323,9 +275,11 @@ namespace GtkSharp.Generation {
 			}
 			
 			GenerateImport (gen_info.Writer);
-			if (comp != null && s_ret == comp.parms.AccessorReturnType)
+			if (comp != null && retval.CSType == comp.Parameters.AccessorReturnType)
 				comp.GenerateImport (gen_info.Writer);
-			
+
+			if (IsDeprecated)
+				gen_info.Writer.WriteLine("\t\t[Obsolete]");
 			gen_info.Writer.Write("\t\t");
 			if (protection != "")
 				gen_info.Writer.Write("{0} ", protection);
@@ -342,7 +296,7 @@ namespace GtkSharp.Generation {
 			
 			if (is_get || is_set)
 			{
-				if (comp != null && s_ret == comp.parms.AccessorReturnType)
+				if (comp != null && retval.CSType == comp.Parameters.AccessorReturnType)
 				{
 					gen_info.Writer.WriteLine ();
 					gen_info.Writer.Write ("\t\t\tset");
@@ -364,49 +318,47 @@ namespace GtkSharp.Generation {
 			StreamWriter sw = gen_info.Writer;
 			sw.WriteLine(" {");
 			if (IsAccessor)
-				body.InitAccessor (sw, sig, indent);
-			body.Initialize(gen_info, is_get, is_set, indent);
+				Body.InitAccessor (sw, Signature, indent);
+			Body.Initialize(gen_info, is_get, is_set, indent);
 
 			SymbolTable table = SymbolTable.Table;
-			IGeneratable ret_igen = table [rettype];
+			IGeneratable ret_igen = table [retval.CType];
 
 			sw.Write(indent + "\t\t\t");
-			if (m_ret == "void") {
-				sw.WriteLine(cname + call + ";");
+			if (retval.MarshalType == "void") {
+				sw.WriteLine(CName + call + ";");
 			} else if (ret_igen is ObjectGen || ret_igen is OpaqueGen) {
-				sw.WriteLine(m_ret + " raw_ret = " + cname + call + ";");
-				sw.WriteLine(indent +"\t\t\t" + s_ret + " ret;");
+				sw.WriteLine(retval.MarshalType + " raw_ret = " + CName + call + ";");
+				sw.WriteLine(indent +"\t\t\t" + retval.CSType + " ret;");
 				sw.WriteLine(indent + "\t\t\tif (raw_ret == IntPtr.Zero)");
 				sw.WriteLine(indent + "\t\t\t\tret = null;");
 				sw.WriteLine(indent + "\t\t\telse");
-				sw.WriteLine(indent +"\t\t\t\tret = " + table.FromNativeReturn(rettype, "raw_ret") + ";");
-			} else if (ret_igen is CustomMarshalerGen) {
-				sw.WriteLine(s_ret + " ret = " + cname + call + ";");
+				sw.WriteLine(indent +"\t\t\t\tret = " + table.FromNativeReturn(retval.CType, "raw_ret" + (retval.Owned ? ", true" : "")) + ";");
 			} else {
-				sw.WriteLine(m_ret + " raw_ret = " + cname + call + ";");
+				sw.WriteLine(retval.MarshalType + " raw_ret = " + CName + call + ";");
 				sw.Write(indent + "\t\t\t");
 				string raw_parms = "raw_ret";
-				if (element_type != null)
-					raw_parms += ", typeof (" + element_type + ")";
-				sw.WriteLine(s_ret + " ret = " + table.FromNativeReturn(rettype, raw_parms) + ";");
+				if (retval.ElementType != String.Empty)
+					raw_parms += ", typeof (" + retval.ElementType + ")";
+				sw.WriteLine(retval.CSType + " ret = " + table.FromNativeReturn(retval.CType, raw_parms) + ";");
 			}
 
-			body.Finish (sw, indent);
-			body.HandleException (sw, indent);
+			Body.Finish (sw, indent);
+			Body.HandleException (sw, indent);
 
-			if (is_get && parms != null) 
-				sw.WriteLine (indent + "\t\t\treturn " + parms.AccessorName + ";");
-			else if (m_ret != "void")
+			if (is_get && Parameters.Count > 0) 
+				sw.WriteLine (indent + "\t\t\treturn " + Parameters.AccessorName + ";");
+			else if (retval.MarshalType != "void")
 				sw.WriteLine (indent + "\t\t\treturn ret;");
 			else if (IsAccessor)
-				body.FinishAccessor (sw, sig, indent);
+				Body.FinishAccessor (sw, Signature, indent);
 
 			sw.Write(indent + "\t\t}");
 		}
 
 		bool IsAccessor { 
 			get { 
-				return s_ret == "void" && sig.IsAccessor; 
+				return retval.CSType == "void" && Signature.IsAccessor; 
 			} 
 		}
 	}
