@@ -28,15 +28,17 @@ namespace GtkSharp.Generation {
 	using System.Text;
 	using System.Xml;
 
-	public class ObjectGen : ClassBase, IGeneratable  {
+	public class ObjectGen : ClassBase  {
 
 		private ArrayList strings = new ArrayList();
 		private ArrayList vm_nodes = new ArrayList();
+		private Hashtable childprops = new Hashtable();
 		private static Hashtable dirs = new Hashtable ();
 
 		public ObjectGen (XmlElement ns, XmlElement elem) : base (ns, elem) 
 		{
 			foreach (XmlNode node in elem.ChildNodes) {
+				string name;
 
 				if (!(node is XmlElement)) continue;
 				XmlElement member = (XmlElement) node;
@@ -55,6 +57,13 @@ namespace GtkSharp.Generation {
 					strings.Add (node);
 					break;
 
+				case "childprop":
+					name = member.GetAttribute ("name");
+					while (childprops.ContainsKey (name))
+						name += "mangled";
+					childprops.Add (name, new ChildProperty (member, this));
+					break;
+
 				default:
 					if (!IsNodeNameHandled (node.Name))
 						Console.WriteLine ("Unexpected node " + node.Name + " in " + CName);
@@ -69,6 +78,12 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		private bool DisableGTypeCtor {
+			get {
+				return Elem.HasAttribute ("disable_gtype_ctor");
+			}
+		}
+
 		private class DirectoryInfo {
 			public string assembly_name;
 			public Hashtable objects;
@@ -99,13 +114,7 @@ namespace GtkSharp.Generation {
 			return result;
 		}
  
-		public void Generate ()
-		{
-			GenerationInfo gen_info = new GenerationInfo (NSElem);
-			Generate (gen_info);
-		}
-
-		public void Generate (GenerationInfo gen_info)
+		public override void Generate (GenerationInfo gen_info)
 		{
 			DirectoryInfo di = GetDirectoryInfo (gen_info.Dir, gen_info.AssemblyName);
 			di.objects.Add (CName, QualifiedName);
@@ -122,7 +131,9 @@ namespace GtkSharp.Generation {
 			SymbolTable table = SymbolTable.Table;
 
 			sw.WriteLine ("#region Autogenerated code");
-			sw.Write ("\tpublic class " + Name);
+			if (IsDeprecated)
+				sw.WriteLine ("\t[Obsolete]");
+			sw.Write ("\tpublic {0} class " + Name, IsAbstract ? "abstract" : "");
 			string cs_parent = table.GetCSType(Elem.GetAttribute("parent"));
 			if (cs_parent != "")
 				sw.Write (" : " + cs_parent);
@@ -138,6 +149,7 @@ namespace GtkSharp.Generation {
 
 			GenCtors (gen_info);
 			GenProperties (gen_info);
+			GenChildProperties (gen_info);
 			
 			bool has_sigs = (sigs != null && sigs.Count > 0);
 			if (!has_sigs && interfaces != null) {
@@ -221,7 +233,10 @@ namespace GtkSharp.Generation {
 			gen_info.Writer.WriteLine("\t\t\tDispose();");
 			gen_info.Writer.WriteLine("\t\t}");
 			gen_info.Writer.WriteLine();
-			gen_info.Writer.WriteLine("\t\tprotected " + Name + "(GLib.GType gtype) : base(gtype) {}");
+			if (!DisableGTypeCtor) {
+				gen_info.Writer.WriteLine("\t\t[Obsolete]");
+				gen_info.Writer.WriteLine("\t\tprotected " + Name + "(GLib.GType gtype) : base(gtype) {}");
+			}
 			gen_info.Writer.WriteLine("\t\tpublic " + Name + "(IntPtr raw) : base(raw) {}");
 			if (ctors.Count == 0 && !DisableVoidCtor) {
 				gen_info.Writer.WriteLine();
@@ -235,6 +250,36 @@ namespace GtkSharp.Generation {
 			base.GenCtors (gen_info);
 		}
 
+		protected void GenChildProperties (GenerationInfo gen_info)
+		{
+			if (childprops.Count == 0)
+				return;
+
+			StreamWriter sw = gen_info.Writer;
+
+			sw.WriteLine ("\t\tpublic class " + Name + "Child : Gtk.Container.ContainerChild {");
+			sw.WriteLine ("\t\t\tprotected internal " + Name + "Child (Gtk.Container parent, Gtk.Widget child) : base (parent, child) {}");
+			sw.WriteLine ("");
+
+			foreach (ChildProperty prop in childprops.Values) {
+				if (prop.Validate ())
+					prop.Generate (gen_info, "\t\t\t");
+				else
+					Console.WriteLine("in Object " + QualifiedName);
+			}
+
+			sw.WriteLine ("\t\t}");
+			sw.WriteLine ("");
+
+			sw.WriteLine ("\t\tpublic override Gtk.Container.ContainerChild this [Gtk.Widget child] {");
+			sw.WriteLine ("\t\t\tget {");
+			sw.WriteLine ("\t\t\t\treturn new " + Name + "Child (this, child);");
+			sw.WriteLine ("\t\t\t}");
+			sw.WriteLine ("\t\t}");
+			sw.WriteLine ("");
+			
+		}
+
 		private void GenVMGlue (GenerationInfo gen_info, XmlElement elem)
 		{
 			StreamWriter sw = gen_info.GlueWriter;
