@@ -3,7 +3,7 @@
 // Author: Mike Kestner <mkestner@speakeasy.net>
 //
 // Copyright (c) 2001-2003 Mike Kestner 
-// Copyright (c) 2003-2004 Novell, Inc.
+// Copyright (c) 2003-2005 Novell, Inc.
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of version 2 of the GNU General Public
@@ -31,6 +31,7 @@ namespace GtkSharp.Generation {
 
 		private string name;
 		private XmlElement elem;
+		private ReturnValue retval;
 		private Parameters parms;
 		private ClassBase container_type;
 		SignalHandler sig_handler;
@@ -38,9 +39,9 @@ namespace GtkSharp.Generation {
 		public Signal (XmlElement elem, ClassBase container_type)
 		{
 			this.elem = elem;
-			this.name = elem.GetAttribute ("name");
-			if (elem["parameters"] != null)
-				parms = new Parameters (elem["parameters"], container_type.NS);
+			name = elem.GetAttribute ("name");
+			retval = new ReturnValue (elem ["return-type"]);
+			parms = new Parameters (elem["parameters"]);
 			this.container_type = container_type;
 			sig_handler = new SignalHandler (elem, container_type.NS);
 		}
@@ -62,7 +63,10 @@ namespace GtkSharp.Generation {
 				return false;
 			}
 			
-			if (parms != null && !parms.Validate ())
+			if (!parms.Validate ())
+				return false;
+
+			if (!retval.Validate ())
 				return false;
 
 			return true;
@@ -116,25 +120,20 @@ namespace GtkSharp.Generation {
 
 		private bool IsVoid {
 			get {
-				return ReturnType == "void";
-			}
-		}
-
-		private string MarshalReturnType {
-			get {
-				string ctype = elem ["return-type"].GetAttribute("type");
-				return SymbolTable.Table.GetMarshalType (ctype);
+				return retval.CSType == "void";
 			}
 		}
 
 		private string ReturnGType {
 			get {
-				ClassBase igen = SymbolTable.Table.GetClassGen (elem ["return-type"].GetAttribute("type"));
+				ClassBase igen = SymbolTable.Table.GetClassGen (retval.CType);
 
 				if (igen is ObjectGen)
 					return "GLib.GType.Object";
+				if (igen is BoxedGen)
+					return retval.CSType + ".GType";
 
-				switch (ReturnType) {
+				switch (retval.CSType) {
 				case "bool":
 					return "GLib.GType.Boolean";
 				case "string":
@@ -142,16 +141,16 @@ namespace GtkSharp.Generation {
 				case "int":
 					return "GLib.GType.Int";
 				default:
-					throw new Exception (ReturnType);
+					throw new Exception (retval.CSType);
 				}
 			}
 		}
 
-		private string ReturnType {
-			get {
-				string ctype = elem ["return-type"].GetAttribute("type");
-				return SymbolTable.Table.GetCSType (ctype);
-			}
+		private bool NeedNew (ClassBase implementor)
+		{
+			return elem.HasAttribute ("new_flag") ||
+				(container_type != null && container_type.GetSignalRecursively (Name) != null) ||
+				(implementor != null && implementor.GetSignalRecursively (Name) != null);
 		}
 
 		public void GenEventHandler (GenerationInfo gen_info)
@@ -171,15 +170,20 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\tpublic delegate void " + EventHandlerName + "(object o, " + EventArgsName + " args);");
 			sw.WriteLine ();
 			sw.WriteLine ("\tpublic class " + EventArgsName + " : GLib.SignalArgs {");
-			if (parms != null) {
-				for (int i = 1; i < parms.Count; i++) {
-					sw.WriteLine ("\t\tpublic " + parms[i].CSType + " " + parms[i].StudlyName + "{");
+			for (int i = 1; i < parms.Count; i++) {
+				sw.WriteLine ("\t\tpublic " + parms[i].CSType + " " + parms[i].StudlyName + "{");
+				if (parms[i].PassAs != "out") {
 					sw.WriteLine ("\t\t\tget {");
 					sw.WriteLine ("\t\t\t\treturn (" + parms[i].CSType + ") Args[" + (i - 1) + "];");
 					sw.WriteLine ("\t\t\t}");
-					sw.WriteLine ("\t\t}");
-					sw.WriteLine ();
 				}
+				if (parms[i].PassAs != "") {
+					sw.WriteLine ("\t\t\tset {");
+					sw.WriteLine ("\t\t\t\tArgs[" + (i - 1) + "] = (" + parms[i].CSType + ")value;");
+					sw.WriteLine ("\t\t\t}");
+				}
+				sw.WriteLine ("\t\t}");
+				sw.WriteLine ();
 			}
 			sw.WriteLine ("\t}");
 			sw.WriteLine ("}");
@@ -190,7 +194,10 @@ namespace GtkSharp.Generation {
 		{
 			VMSignature vmsig = new VMSignature (parms);
 			sw.WriteLine ("\t\t[GLib.DefaultSignalHandler(Type=typeof(" + (implementor != null ? implementor.QualifiedName : container_type.QualifiedName) + "), ConnectionMethod=\"Override" + Name +"\")]");
-			sw.WriteLine ("\t\tprotected virtual {0} {1} ({2})", ReturnType, "On" + Name, vmsig.ToString ());
+			sw.Write ("\t\tprotected ");
+			if (NeedNew (implementor))
+				sw.Write ("new ");
+			sw.WriteLine ("virtual {0} {1} ({2})", retval.CSType, "On" + Name, vmsig.ToString ());
 			sw.WriteLine ("\t\t{");
 			if (IsVoid)
 				sw.WriteLine ("\t\t\tGLib.Value ret = GLib.Value.Empty;");
@@ -217,8 +224,10 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t\tg_signal_chain_from_overridden (inst_and_params.ArrayPtr, ref ret);");
 			if (cleanup != "")
 				sw.WriteLine (cleanup);
+			sw.WriteLine ("\t\t\tforeach (GLib.Value v in vals)");
+			sw.WriteLine ("\t\t\t\tv.Dispose ();");
 			if (!IsVoid)
-				sw.WriteLine ("\t\t\treturn (" + ReturnType + ") ret;");
+				sw.WriteLine ("\t\t\treturn (" + retval.CSType + ") ret;");
 			sw.WriteLine ("\t\t}\n");
 		}
 
@@ -226,13 +235,17 @@ namespace GtkSharp.Generation {
 		{
 			ImportSignature isig = new ImportSignature (parms, container_type.NS);
 			ManagedCallString call = new ManagedCallString (parms);
-			sw.WriteLine ("\t\tdelegate " + MarshalReturnType + " " + Name + "Delegate (" + isig.ToString () + ");\n");
+			sw.WriteLine ("\t\tdelegate " + retval.ToNativeType + " " + Name + "Delegate (" + isig.ToString () + ");\n");
 			sw.WriteLine ("\t\tstatic {0} {1};\n", Name + "Delegate", Name + "Callback");
-			sw.WriteLine ("\t\tstatic " + MarshalReturnType + " " + Name.ToLower() + "_cb (" + isig.ToString () + ")");
+			sw.WriteLine ("\t\tstatic " + retval.ToNativeType + " " + Name.ToLower() + "_cb (" + isig.ToString () + ")");
 			sw.WriteLine ("\t\t{");
 			sw.WriteLine ("\t\t\t{0} obj = GLib.Object.GetObject ({1}, false) as {0};", implementor != null ? implementor.Name : container_type.Name, parms[0].Name);
-			sw.Write ("\t\t\t{0}", IsVoid ? "" : "return ");
+			sw.Write (call.Setup ("\t\t\t"));
+			sw.Write ("\t\t\t{0}", IsVoid ? "" : retval.CSType == retval.ToNativeType ? "return " : retval.CSType + " raw_ret = ");
 			sw.WriteLine ("obj.{0} ({1});", "On" + Name, call.ToString ());
+			sw.Write (call.Finish ("\t\t\t"));
+			if (!IsVoid && retval.CSType != retval.ToNativeType)
+				sw.WriteLine ("\t\t\treturn {0};", SymbolTable.Table.ToNativeReturn (retval.CType, "raw_ret"));
 			sw.WriteLine ("\t\t}\n");
 			string cname = "\"" + elem.GetAttribute("cname") + "\"";
 			sw.WriteLine ("\t\tprivate static void Override" + Name + " (GLib.GType gtype)");
@@ -261,21 +274,21 @@ namespace GtkSharp.Generation {
 
 			sw.WriteLine("\t\t[GLib.Signal("+ cname + ")]");
 			sw.Write("\t\tpublic ");
-			if (elem.HasAttribute("new_flag") || (container_type != null && container_type.GetSignalRecursively (Name) != null) || (implementor != null && implementor.GetSignalRecursively (Name) != null))
+			if (NeedNew (implementor))
 				sw.Write("new ");
 			sw.WriteLine("event " + EventHandlerQualifiedName + " " + Name + " {");
 			sw.WriteLine("\t\t\tadd {");
 			sw.WriteLine("\t\t\t\tif (value.Method.GetCustomAttributes(typeof(GLib.ConnectBeforeAttribute), false).Length > 0) {");
 			sw.WriteLine("\t\t\t\t\tif (BeforeHandlers[" + cname + "] == null)");
 			sw.Write("\t\t\t\t\t\tBeforeSignals[" + cname + "] = new " + qual_marsh);
-			sw.Write("(this, " + cname + ", value, typeof (" + EventArgsQualifiedName + "), 0);");
+			sw.WriteLine("(this, " + cname + ", value, typeof (" + EventArgsQualifiedName + "), 0);");
 			sw.WriteLine("\t\t\t\t\telse");
 			sw.WriteLine("\t\t\t\t\t\t((GLib.SignalCallback) BeforeSignals [{0}]).AddDelegate (value);", cname);
 			sw.WriteLine("\t\t\t\t\tBeforeHandlers.AddHandler(" + cname + ", value);");
 			sw.WriteLine("\t\t\t\t} else {");
 			sw.WriteLine("\t\t\t\t\tif (AfterHandlers[" + cname + "] == null)");
 			sw.Write("\t\t\t\t\t\tAfterSignals[" + cname + "] = new " + qual_marsh);
-			sw.Write("(this, " + cname + ", value, typeof (" + EventArgsQualifiedName + "), 1);");
+			sw.WriteLine("(this, " + cname + ", value, typeof (" + EventArgsQualifiedName + "), 1);");
 			sw.WriteLine("\t\t\t\t\telse");
 			sw.WriteLine("\t\t\t\t\t\t((GLib.SignalCallback) AfterSignals [{0}]).AddDelegate (value);", cname);
 			sw.WriteLine("\t\t\t\t\tAfterHandlers.AddHandler(" + cname + ", value);");
