@@ -68,7 +68,7 @@ namespace GLib {
 		// [native pointer size] * [count] bytes.
 
 		[DllImport("libglib-2.0-0.dll")]
-		static extern IntPtr g_malloc(ulong size);
+		static extern IntPtr g_malloc(UIntPtr size);
 
 		static bool check_sixtyfour () {
 			int szint = Marshal.SizeOf (typeof (int));
@@ -90,8 +90,8 @@ namespace GLib {
 			for (int i = 0; i < args.Length; i++)
 				ptrs[i] = (int) Marshal.StringToHGlobalAuto (args[i]);
 
-			IntPtr buf = g_malloc ((ulong) Marshal.SizeOf(typeof(int)) * 
-					       (ulong) args.Length);
+			IntPtr buf = g_malloc (new UIntPtr ((ulong) Marshal.SizeOf(typeof(int)) * 
+					       (ulong) args.Length));
 			Marshal.Copy (ptrs, 0, buf, ptrs.Length);
 			return buf;
 		}
@@ -103,8 +103,8 @@ namespace GLib {
 			for (int i = 0; i < args.Length; i++)
 				ptrs[i] = (long) Marshal.StringToHGlobalAuto (args[i]);
 				
-			IntPtr buf = g_malloc ((ulong) Marshal.SizeOf(typeof(long)) * 
-					       (ulong) args.Length);
+			IntPtr buf = g_malloc (new UIntPtr ((ulong) Marshal.SizeOf(typeof(long)) * 
+					       (ulong) args.Length));
 			Marshal.Copy (ptrs, 0, buf, ptrs.Length);
 			return buf;
 		}
@@ -161,7 +161,7 @@ namespace GLib {
 		}
 
 		static DateTime local_epoch = new DateTime (1970, 1, 1, 0, 0, 0);
-		static int utc_offset = (int) (DateTime.Now.Subtract (DateTime.UtcNow).TotalSeconds);
+		static int utc_offset = (int) (TimeZone.CurrentTimeZone.GetUtcOffset (DateTime.Now)).TotalSeconds;
 
 		public static IntPtr DateTimeTotime_t (DateTime time)
 		{
@@ -178,6 +178,9 @@ namespace GLib {
 
 		public static char GUnicharToChar (uint ucs4_char)
 		{ 
+			if (ucs4_char == 0)
+				return (char) 0;
+
 			IntPtr raw_ret = gtksharp_unichar_to_utf8_string (ucs4_char);
 			string ret = GLib.Marshaller.PtrToStringGFree(raw_ret);
 			if (ret.Length > 1)
@@ -187,11 +190,11 @@ namespace GLib {
 		}
 
 		[DllImport("glibsharpglue")]
-		static extern uint glibsharp_utf16_to_gunichar (ushort c);
+		static extern uint glibsharp_utf16_to_unichar (ushort c);
 
 		public static uint CharToGUnichar (char c)
 		{
-			return glibsharp_utf16_to_gunichar ((ushort) c);
+			return glibsharp_utf16_to_unichar ((ushort) c);
 		}
 
 	}
