@@ -67,6 +67,12 @@ GdkColor *gtksharp_gtk_style_get_fg (GtkStyle *style, int i);
 
 GdkColor *gtksharp_gtk_style_get_bg (GtkStyle *style, int i);
 
+GdkColor *gtksharp_gtk_style_get_light (GtkStyle *style, int i);
+
+GdkColor *gtksharp_gtk_style_get_mid (GtkStyle *style, int i);
+
+GdkColor *gtksharp_gtk_style_get_dark (GtkStyle *style, int i);
+
 GdkColor *gtksharp_gtk_style_get_text (GtkStyle *style, int i);
 
 GdkColor *gtksharp_gtk_style_get_base (GtkStyle *style, int i);
@@ -76,6 +82,11 @@ PangoFontDescription *gtksharp_gtk_style_get_font_description (GtkStyle *style);
 int gtksharp_gtk_style_get_thickness (GtkStyle *style, int x);
 
 void gtksharp_gtk_style_set_thickness (GtkStyle *style, int thickness);
+
+GdkPixmap *gtksharp_gtk_style_get_bg_pixmap (GtkStyle *style, int i);
+
+void gtksharp_gtk_style_set_bg_pixmap (GtkStyle *style, int i, GdkPixmap *pixmap);
+
 /* */
 
 /* FIXME: include all fields */
@@ -223,12 +234,30 @@ gtksharp_gtk_style_get_bg (GtkStyle *style, int i)
 	return &style->bg[i];
 }
 
+GdkColor*
+gtksharp_gtk_style_get_light (GtkStyle *style, int i)
+{
+	return &style->light[i];
+}
+
 GdkColor*
 gtksharp_gtk_style_get_mid (GtkStyle *style, int i)
 {
 	return &style->mid[i];
 }
 
+GdkColor*
+gtksharp_gtk_style_get_dark (GtkStyle *style, int i)
+{
+	return &style->dark[i];
+}
+
+GdkColor*
+gtksharp_gtk_style_get_text (GtkStyle *style, int i)
+{
+	return &style->text[i];
+}
+
 GdkColor*
 gtksharp_gtk_style_get_base (GtkStyle *style, int i)
 {
@@ -258,3 +287,16 @@ gtksharp_gtk_style_set_thickness (GtkStyle *style, int thickness)
 	else
 		style->ythickness = -thickness;
 }
+
+GdkPixmap *
+gtksharp_gtk_style_get_bg_pixmap (GtkStyle *style, int i)
+{
+	return style->bg_pixmap[i];
+}
+
+void gtksharp_gtk_style_set_bg_pixmap (GtkStyle *style, int i, GdkPixmap *pixmap)
+{
+	g_object_ref (G_OBJECT (pixmap));
+	style->bg_pixmap[i] = pixmap;
+}
+
